import _objectSpread from "@babel/runtime/helpers/objectSpread";
import React from 'react';
import ReactDOMServer from 'react-dom/server';
import withContext from '@s-ui/hoc/lib/withContext';

var EMPTY_FUNC = function EMPTY_FUNC() {
  return Promise.resolve({});
};

var hrTimeToMs = function hrTimeToMs(diff) {
  return diff[0] * 1e3 + diff[1] * 1e-6;
};

export default function ssrComponentWithInitialProps(_ref) {
  var Target = _ref.Target,
      context = _ref.context,
      renderProps = _ref.renderProps,
      _ref$useStream = _ref.useStream,
      useStream = _ref$useStream === void 0 ? false : _ref$useStream;
  var startGetInitialProps = process.hrtime(); // get the page with the initialProps

  var pageComponent = renderProps.components[renderProps.components.length - 1]; // use the getInitialProps from the page to retrieve the props to initialize

  var getInitialProps = pageComponent.getInitialProps || EMPTY_FUNC;
  return getInitialProps(context).then(function (initialProps) {
    var diffGetInitialProps = process.hrtime(startGetInitialProps); // create the App component with the context, the initialProps and the Target Component

    var App = withContext(_objectSpread({}, context, {
      initialProps: initialProps
    }))(Target);

    if (useStream) {
      var reactStream = ReactDOMServer.renderToNodeStream(React.createElement(App, Object.assign({}, renderProps, {
        initialProps: initialProps
      })));
      return {
        initialProps: initialProps,
        reactStream: reactStream,
        performance: {
          getInitialProps: hrTimeToMs(diffGetInitialProps)
        }
      };
    } else {
      // start to calculate renderToString
      var startRenderToString = process.hrtime(); // render our whole application with the needed props and get the html string

      var reactString = ReactDOMServer.renderToString(React.createElement(App, Object.assign({}, renderProps, {
        initialProps: initialProps
      })));
      var diffRenderToString = process.hrtime(startRenderToString);
      return {
        initialProps: initialProps,
        reactString: reactString,
        performance: {
          getInitialProps: hrTimeToMs(diffGetInitialProps),
          renderToString: hrTimeToMs(diffRenderToString)
        }
      };
    }
  });
}